/*  Example implementation of a merge sort
    Copyright (C) 2002  Clinton Ebadi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdlib.h>
#include "algorithm.h"

/* merges the two parts of data front1..back1 and front2..back2 into
   front1..back2. Make sure front1..back2 is continuous! */
static void merge (void* data[], cmp_func cmp, int front1, int back1, 
		   int front2, int back2);

/* real merge sort caller */
static void _merge_sort (void* data[], cmp_func cmp, int front, int back);

void
merge_sort (void* data[], cmp_func cmp, unsigned int sz)
{
  _merge_sort (data, cmp, 0, sz - 1);
}

void
_merge_sort (void* data[], cmp_func cmp, int front, int back)
{
  if (front < back)
    {
      int middle = (front + back) / 2;

      _merge_sort (data, cmp, front, middle);
      _merge_sort (data, cmp, middle + 1, back);

      merge (data, cmp, front, middle, middle+1, back);
    }
}

void
merge (void* data[], cmp_func cmp, int front1, int back1, 
       int front2, int back2)
{
  void **temp_array;
  int saved = front1;
  int temp_array_idx = 0;

  temp_array = malloc ((back2 - front1) * sizeof (void*) + sizeof (void*));
  
  while ( (front1 <= back1) && (front2 <= back2) )
    {
      if ( cmp (data[front1], data[front2]) == -1 )
	{
	  temp_array[temp_array_idx] = data[front1];
	  ++front1;
	}
      else
	{
	  temp_array[temp_array_idx] = data[front2];
	  ++front2;
	}
      ++temp_array_idx;
    }

  while (front1 <= back1)
    {
      temp_array[temp_array_idx] = data[front1];
      ++front1;
      ++temp_array_idx;
    }

  while (front2 <= back2)
    {
      temp_array[temp_array_idx] = data[front2];
      ++front2;
      ++temp_array_idx;
    }

  for (temp_array_idx = 0; saved <= back2; saved++, temp_array_idx++)
    data[saved] = temp_array[temp_array_idx];

  return;
}
