/*  Example implementation of a Primality Test
    Copyright (C) 2002  Clinton Ebadi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "algorithm.h"

/* find_div finds a divisor of x that is:
   guess <= divisor <= sqrt (x) */
static int find_div (int x, int guess);
static int next (int x); /* returns 3 if x == 2, x + 2 otherwise */

int
prime (int x)
{
  return (x == find_div (x, 2));
}

int
find_div (int x, int guess)
{
  while (1)
    {
      if ((guess * guess) > x) /* guess*guess = guess^2 */
	return x;
      else if ((x % guess) == 0)
	return guess;
      else
	guess = next (guess);
    }
}

int next (int x)
{
  if (x == 2)
    return 3;
  else
    return x + 2;
}
