/*  Hash Table Data Structure Example
    Copyright (C) 2001,2002  Clinton Ebadi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "hash_table.h"
#include <stdlib.h>
#include <assert.h>
#include <glib.h>

hash_table*
make_hash_table (hash_func h, copy_func c, eq_func e, size_t s)
{
  int i;  
  hash_table* temp = malloc (sizeof (hash_table));

  temp->hash = h;
  temp->copy = c;
  temp->ecmp = e;
  temp->size = s;

  if (s == 0) /* cannot have empty table */
    s = 10;

  temp->data = malloc (s * sizeof (GSList*));
  for (i=0;i<s;i++) /* set all lists to empty lists */
    temp->data[i] = NULL;

  return temp;

}

void
destroy_hash_table (hash_table* tbl)
{
  int i;

  assert (tbl != NULL);

  for (i=0;i<tbl->size;i++) /* clear hash table */
    g_slist_free (tbl->data[i]);

  free (tbl->data);
  free (tbl);
  tbl = NULL;

}

void* 
hash_table_get (hash_table* tbl, void* dat)
{
  size_t loc;
  GSList* tmp_list;

  assert (tbl != NULL);

  loc = tbl->hash (dat, tbl->size);
  assert (loc < tbl->size);

  if (tbl->data[loc] == NULL)
    return NULL; /* oops, no data in this bucket yet */

  tmp_list = g_slist_find_custom (tbl->data[loc],dat,tbl->ecmp);

  if (tmp_list == NULL)
    return NULL;

  return tbl->copy (tmp_list->data);

}

void
hash_table_put (hash_table* tbl, void* dat)
{
  size_t loc;
  assert (tbl != NULL);

  loc = tbl->hash (dat,tbl->size);
  assert (loc < tbl->size);

  /* put copy of dat into hash table at front of list  */
  tbl->data[loc] = g_slist_prepend (tbl->data[loc], tbl->copy (dat));

}

void
hash_table_remove (hash_table* tbl, void* dat)
{
  size_t loc;
  GSList *temp_list;
  assert (tbl != NULL);

  loc = tbl->hash (dat,tbl->size);
  assert (loc < tbl->size);

  temp_list = g_slist_find_custom (tbl->data[loc],dat,tbl->ecmp);
  if (temp_list)
    {
      tbl->data[loc] = g_slist_remove_link (tbl->data[loc],temp_list);
      g_slist_free_1 (temp_list);
    }


}
