/*  Hash Table Data Structure Example
    Copyright (C) 2001,2002  Clinton Ebadi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* test program for hash_table.
 * Uses a simple hash table of strings
 */

#include "hash_table.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* hash and copy functions for strings */
static size_t str_hash (void* data, size_t sz);
static void*  str_copy (void* data);
static gint   str_eq (const void* dat1, const void* dat2);

/* macro to put a string into the hash table AND print it */
#define hash_table_put_print(tbl, str) \
        hash_table_put (tbl, str); \
        printf ("Hashed: %s\n", str);

int main (int argc, char *argv[])
{
  hash_table* table;
  char* temp_name = NULL;

  table = make_hash_table (&str_hash, &str_copy, &str_eq, 10);

  /* my handle, the handles of a few others people and some bots...  I
     needed some strings to hash, and I had these handy so...I know
     that I could have read these from a file, but that would have
     obfusicated the code some.
  */
  printf ("Filling Tables with seed data (people from IRC)\n");
  hash_table_put_print (table, "unknown_lamer");
  hash_table_put_print (table, "CowBot");
  hash_table_put_print (table, "ZugBot");
  hash_table_put_print (table, "tim_m");
  hash_table_put_print (table, "mike_home");
  hash_table_put_print (table, "Cathy");
  hash_table_put_print (table, "She-Ra");
  hash_table_put_print (table, "grok_grok");
  hash_table_put_print (table, "cdlu");
  hash_table_put_print (table, "roblimo");
  hash_table_put_print (table, "r_g");
  hash_table_put_print (table, "ray_"); /* the '_' is not a typo */
  hash_table_put_print (table, "emacsen");
  hash_table_put_print (table, "Andypoo");
  hash_table_put_print (table, "Andycow");

  /* ok, that should be enough data to fill the table and spill into
     the lists, now is time to look around in the list and do stuff */

  printf ("Searching for roblimo...");
  temp_name = hash_table_get (table, "roblimo");
  if (temp_name)
    {
      printf ("FOUND: %s\n",temp_name);
      free (temp_name); /* data is always copied */
      temp_name = NULL;
    }
  else
    printf ("roblimo not found\n");

  printf ("Searching for greycat...");
  temp_name = hash_table_get (table, "greycat");

  if (temp_name)
    {
      printf ("FOUND: %s",temp_name);
      free (temp_name);
      temp_name = NULL;
    }
  else
    printf ("greycat not found (wrong channel!)\n");

  printf ("No one likes roblimo anymore, deleting from table\n");
  hash_table_remove (table, "roblimo");
  printf ("Deleted, making sure he is gone\n");
  temp_name = hash_table_get (table, "roblimo");
  if (temp_name)
    {
      printf ("Still in the table..he must have more than one entry\n");
      free (temp_name);
      temp_name = NULL;
    }
  else
    printf ("Yep, gone for good.\n");

  destroy_hash_table (table);
  return 0;
}

/* definitions of the string hash functions */

size_t
str_hash (void* data, size_t sz)
{
  /* adds ASCII values of first two ASCII chars and mods them with the
     size of the table
  */

  int temp;
  char *t, a, b;
  t = (char *) data;
  a = t[0];
  b = t[strlen (t)-1];

  temp = (int) a + b;

  return temp % sz;
}

void*
str_copy (void* data)
{
  char* temp = strdup (data); /* duplicate string */
  return temp;
}

gint 
str_eq (const void* dat1, const void* dat2)
{
  int t = strcmp ((const char *) dat1, (const char *) dat2);

  if (t != 0)
    return 1;

  return 0;
}
