/*  Stack Data Structure Example
    Copyright (C) 2001  Clinton Ebadi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/* -*- C abbrev -*- */

#include <assert.h> /* assertions */
#include "stack.h"

stack*
make_stack (size_t sz)
{
  stack* temp =  (stack *) malloc (1 * sizeof (stack));
  if (sz == 0) /* user tried to create a null stack */
    {
      temp->data = malloc (100 * sizeof (void**));
      temp->size = 100;
    }
  else
    {
      temp->data = malloc (sz * sizeof (void**)) ;
      temp->size = sz;
    }

  temp->loc = -1;
  return temp;
}

void*
stack_peak (stack* stk)
{
  assert (stk != NULL && stk->loc != -1);
  return (stk->data[stk->loc]);
}

void
stack_pop (stack* stk)
{
  assert (stk != NULL && stk->loc != -1);
  --stk->loc;
}

void
stack_push (stack* stk, void* data)
{
  assert (stk != NULL);

  if (stk->loc == stk->size)
    {
      stk->data = realloc (stk->data, (stk->size * 2) * sizeof (void**));
      stk->size = stk->size * 2;
    }

  stk->data[stk->loc+1] = data;
  ++stk->loc;
}

void
stack_destroy (stack* stk)
{
  assert (stk != NULL && stk->data != NULL);
  free (stk->data);
  free (stk);
  stk = NULL;
}
