;    Merge Sort Algorithm Example
;    Copyright (C) 2002  Clinton Ebadi
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


;; TODO:
;; - Make a few of the commonly used stuff macros for readability

;; merge-sort vec: vector to search in, cmp: comparison function 
;; modifies vec in place

(load "vswap.scm")

(define merge
  (lambda (vec cmp front1 back1 front2 back2)
    (let ((saved front1)
	  (temp-vec-idx 0)
	  (temp-vec (make-vector (+ (- back2 front1) 1))))
      (let copy-loop ((fr1 front1)
		      (fr2 front2))
	(set! front1 fr1)
	(set! front2 fr2)
	(if (and (<= fr1 back1) (<= fr2 back2))
	    (if (< (cmp (vector-ref vec fr1) (vector-ref vec fr2)) 0)
		(begin
		  (vector-set! temp-vec temp-vec-idx (vector-ref vec fr1))
		  (set! temp-vec-idx (+ temp-vec-idx 1))
		  (copy-loop (+ fr1 1) fr2))
		(begin
		  (vector-set! temp-vec temp-vec-idx (vector-ref vec fr2))
		  (set! temp-vec-idx (+ temp-vec-idx 1))
		  (copy-loop fr1 (+ fr2 1))))))

      ;; copy remaining elements from left side into temp-vec
      (do ((front1 front1 (+ front1 1))
	   (temp-vec-idx temp-vec-idx (+ temp-vec-idx 1)))
	  ((> front1 back1))
	(vector-set! temp-vec temp-vec-idx (vector-ref vec front1)))
      ;; copy remaining elements from right side into temp-vec
      (do ((front2 front2 (+ front2 1))
	   (temp-vec-idx temp-vec-idx (+ temp-vec-idx 1)))
	  (> front2 back2)
	(vector-set! temp-vec temp-vec-idx (vector-ref vec front2)))

      ;; copy temp-vec back into vec
      (do ((temp-vec-idx 0 (+ temp-vec-idx 1))
	   (saved saved (+ saved 1)))
	  ((> saved back2))
	(vector-set! vec saved (vector-ref temp-vec temp-vec-idx))))))

(define _merge_sort ; designed to be called recursively
  (lambda (vec cmp front back)
    (if (< front back)
	(let ((middle (inexact->exact (floor (/ (+ front back) 2)))))
	  (_merge_sort vec cmp front middle)
	  (_merge_sort vec cmp (+ middle 1) back)
	  (merge vec cmp front middle (+ middle 1) back)))))

(define merge-sort
  (lambda (vec cmp)
    (if (not (vector? vec))
	(throw 'type-error))
    (_merge_sort vec cmp 0 (- (vector-length vec) 1))))
