;    Quick Sort Algorithm Example
;    Copyright (C) 2002  Clinton Ebadi
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


;; quick-sort vec: vector to search in, cmp: comparison function 
;; modifies vec in place

(load "vswap.scm")

(define quick-sort
  (lambda (vec cmp)
    (_quick-sort vec cmp 0 (- (vector-length vec) 1))))

(define _quick-sort
  (lambda (vec cmp first last)
    (if (< first last)
      (let ((split-point (split vec cmp first last)))
	(_quick-sort vec cmp first (- split-point 1))
	(_quick-sort vec cmp (+ split-point 1) last)))))

;;; split splits a vector into two pieces along a split value it then
;;; returns the split value to the caller so the split vector can be
;;; further sorted.
(define split
  (lambda (vec cmp first last)
    (let ((splitval (vector-ref vec first))
	  (saved first))
      (set! first (+ 1 first)) ; sort everything beyond the split value
      (let swap-loop ; this moves first towards last and vice versa
	  ((break #f)) ; if #t break from loop
	(cond
	  ((not break) ; if not time to break...
	   (let move-to-last ((break break)) ; moves first towards last
	     (if (not break)
	       (if (> (cmp (vector-ref vec first) splitval) 0)
		 (move-to-last #t)
		 (begin 
		   (set! first (+ 1 first))
		   ;; sets break to !(first <= last)
		   (move-to-last (> (cmp first last) 0))))))
	     (set! break (> (cmp first last) 0))
	     (let move-to-first ((break break)) ; moves last towards first
	       (if (not break)
		 ;; if (last <= splitval)
		 (if (not (> (cmp (vector-ref vec last) splitval) 0))
		   (move-to-first #t)
		   (begin 
		     (set! last (- last 1))
		     ;; sets break to !(first <= last)
		     (move-to-first (> (cmp first last) 0))))))
	     (cond ((< first last)
		    (vswap vec first last)
		    (set! first (+ 1 first))
		    (set! last (- last 1))))
	     (swap-loop (> first last)))))
      (vswap vec saved last) ; told you we would use saved
      last )))